@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.3.2.  */
+/* A Bison parser, made by GNU Bison 3.7.5.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -31,8 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* Undocumented macros, especially those whose name start with YY_,
-   are private implementation details.  Do not rely on them.  */
+/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
+   especially those whose name start with YY_ or yy_.  They are
+   private implementation details that can be changed or removed.  */
 
 #ifndef YY_YY_PARSER_H_INCLUDED
 # define YY_YY_PARSER_H_INCLUDED
@@ -44,49 +45,58 @@
 extern int yydebug;
 #endif
 
-/* Token type.  */
+/* Token kinds.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
   enum yytokentype
   {
-    T_id = 258,
-    T_str = 259,
-    T_intval = 260,
-    T_true = 261,
-    T_false = 262,
-    T_initial = 263,
-    T_inputs = 264,
-    T_actions = 265,
-    T_size = 266,
-    T_dumpdot = 267,
-    T_autoreorder = 268,
-    T_reorder = 269,
-    T_win2 = 270,
-    T_win2ite = 271,
-    T_sift = 272,
-    T_siftite = 273,
-    T_none = 274,
-    T_cache = 275,
-    T_tautology = 276,
-    T_print = 277,
-    T_lpar = 278,
-    T_rpar = 279,
-    T_equal = 280,
-    T_semi = 281,
-    T_dot = 282,
-    T_exist = 283,
-    T_forall = 284,
-    T_biimp = 285,
-    T_imp = 286,
-    T_or = 287,
-    T_nor = 288,
-    T_xor = 289,
-    T_nand = 290,
-    T_and = 291,
-    T_not = 292
+    YYEMPTY = -2,
+    YYEOF = 0,                     /* "end of file"  */
+    YYerror = 256,                 /* error  */
+    YYUNDEF = 257,                 /* "invalid token"  */
+    T_id = 258,                    /* T_id  */
+    T_str = 259,                   /* T_str  */
+    T_intval = 260,                /* T_intval  */
+    T_true = 261,                  /* T_true  */
+    T_false = 262,                 /* T_false  */
+    T_initial = 263,               /* T_initial  */
+    T_inputs = 264,                /* T_inputs  */
+    T_actions = 265,               /* T_actions  */
+    T_size = 266,                  /* T_size  */
+    T_dumpdot = 267,               /* T_dumpdot  */
+    T_autoreorder = 268,           /* T_autoreorder  */
+    T_reorder = 269,               /* T_reorder  */
+    T_win2 = 270,                  /* T_win2  */
+    T_win2ite = 271,               /* T_win2ite  */
+    T_sift = 272,                  /* T_sift  */
+    T_siftite = 273,               /* T_siftite  */
+    T_none = 274,                  /* T_none  */
+    T_cache = 275,                 /* T_cache  */
+    T_tautology = 276,             /* T_tautology  */
+    T_print = 277,                 /* T_print  */
+    T_lpar = 278,                  /* T_lpar  */
+    T_rpar = 279,                  /* T_rpar  */
+    T_equal = 280,                 /* T_equal  */
+    T_semi = 281,                  /* T_semi  */
+    T_dot = 282,                   /* T_dot  */
+    T_exist = 283,                 /* T_exist  */
+    T_forall = 284,                /* T_forall  */
+    T_biimp = 285,                 /* T_biimp  */
+    T_imp = 286,                   /* T_imp  */
+    T_or = 287,                    /* T_or  */
+    T_nor = 288,                   /* T_nor  */
+    T_xor = 289,                   /* T_xor  */
+    T_nand = 290,                  /* T_nand  */
+    T_and = 291,                   /* T_and  */
+    T_not = 292                    /* T_not  */
   };
+  typedef enum yytokentype yytoken_kind_t;
 #endif
-/* Tokens.  */
+/* Token kinds.  */
+#define YYEMPTY -2
+#define YYEOF 0
+#define YYerror 256
+#define YYUNDEF 257
 #define T_id 258
 #define T_str 259
 #define T_intval 260
