@@ -1,5 +1,5 @@
 /*========================================================================
-	       Copyright (C) 1996-2003 by Jorn Lind-Nielsen
+	       Copyright (C) 1996-2003, 2021 by Jorn Lind-Nielsen
 			    All rights reserved
 
     Permission is hereby granted, without written agreement and without
@@ -127,6 +127,14 @@ typedef struct s_bddPair
    struct s_bddPair *next;
 } bddPair;
 
+typedef struct
+{
+  int *vars;
+  int *stacktop;
+  int *stack;
+  BDD fun;
+  int nvars;
+} mintermEnumerator;
 
 /*=== Status information ===============================================*/
 
@@ -327,12 +335,18 @@ BUDDY_API int      bdd_setbddpair(bddPair*, int, BDD);
 BUDDY_API int      bdd_setbddpairs(bddPair*, int*, BDD*, int);
 BUDDY_API void     bdd_resetpair(bddPair *);
 BUDDY_API void     bdd_freepair(bddPair*);
+BUDDY_API int      bdd_stable_cmp(BDD, BDD);
 
   /* In bddop.c */
 
 BUDDY_API int      bdd_setcacheratio(int);
 BUDDY_API BDD      bdd_buildcube(int, int, BDD *);
 BUDDY_API BDD      bdd_ibuildcube(int, int, int *);
+BUDDY_API BDD      bdd_ibuildcube2(int, const int *);
+BUDDY_API mintermEnumerator* bdd_init_minterm(BDD, BDD);
+BUDDY_API int      bdd_first_minterm(mintermEnumerator*);
+BUDDY_API int      bdd_next_minterm(mintermEnumerator*);
+BUDDY_API void     bdd_free_minterm(mintermEnumerator*);
 BUDDY_API BDD      bdd_not(BDD);
 BUDDY_API BDD      bdd_apply(BDD, BDD, int);
 BUDDY_API BDD      bdd_and(BDD, BDD);
@@ -365,6 +379,7 @@ BUDDY_API BDD      bdd_support(BDD);
 BUDDY_API BDD      bdd_satone(BDD);
 BUDDY_API BDD      bdd_satoneset(BDD, BDD, BDD);
 BUDDY_API BDD      bdd_fullsatone(BDD);
+BUDDY_API BDD      bdd_satoneshortest(BDD, unsigned, unsigned, unsigned);
 BUDDY_API BDD	bdd_satprefix(BDD *);
 BUDDY_API void     bdd_allsat(BDD r, bddallsathandler handler);
 BUDDY_API double   bdd_satcount(BDD);
@@ -375,7 +390,8 @@ BUDDY_API int      bdd_nodecount(BDD);
 BUDDY_API int      bdd_anodecount(BDD *, int);
 BUDDY_API int*     bdd_varprofile(BDD);
 BUDDY_API double   bdd_pathcount(BDD);
-
+BUDDY_API int      bdd_have_common_assignment(BDD, BDD);
+BUDDY_API int      bdd_is_cube(BDD);
 
 /* In file "bddio.c" */
 
@@ -580,9 +596,15 @@ protected:
    friend bdd      bdd_makesetpp(int *, int);
    friend int      bdd_setbddpair(bddPair*, int, const bdd &);
    friend int      bdd_setbddpairs(bddPair*, int*, const bdd *, int);
+   friend int      bdd_stable_cmp(const bdd&, const bdd&);
    friend bdd      bdd_from_int(int i);
    friend bdd      bdd_buildcube(int, int, const bdd *);
    friend bdd      bdd_ibuildcubepp(int, int, int *);
+   friend bdd      bdd_ibuildcubepp2(int, const int *);
+   friend mintermEnumerator* bdd_init_minterm(const bdd&, const bdd&);
+   friend int      bdd_first_mintermpp(mintermEnumerator*);
+   friend int      bdd_next_mintermpp(mintermEnumerator*);
+   friend void     bdd_free_mintermpp(mintermEnumerator*);
    friend bdd      bdd_not(const bdd &);
    friend bdd      bdd_simplify(const bdd &, const bdd &);
    friend bdd      bdd_apply(const bdd &, const bdd &, int);
@@ -615,6 +637,8 @@ protected:
    friend bdd      bdd_satone(const bdd &);
    friend bdd      bdd_satoneset(const bdd &, const bdd &, const bdd &);
    friend bdd      bdd_fullsatone(const bdd &);
+   friend bdd      bdd_satoneshortest(const bdd &,
+                                      unsigned, unsigned, unsigned);
    friend bdd      bdd_satprefix(bdd &);
    friend void     bdd_allsat(const bdd &r, bddallsathandler handler);
    friend void     bdd_allsat(const bdd &r, bddallsathandler_old handler);
@@ -626,6 +650,8 @@ protected:
    friend int      bdd_anodecountpp(const bdd *, int);
    friend int*     bdd_varprofile(const bdd &);
    friend double   bdd_pathcount(const bdd &);
+   friend int      bdd_have_common_assignment(const bdd&, const bdd&);
+   friend int      bdd_is_cube(const bdd&);
 
    friend void   bdd_fprinttable(FILE *, const bdd &);
    friend void   bdd_printtable(const bdd &);
@@ -724,6 +750,9 @@ inline bdd bdd_makesetpp(int *v, int n)
 inline int bdd_setbddpair(bddPair *p, int ov, const bdd &nv)
 { return bdd_setbddpair(p,ov,nv.root); }
 
+inline int bdd_stable_cmp(const bdd&l, const bdd& r)
+{ return bdd_stable_cmp(l.root, r.root); }
+
 inline bdd bdd_from_int(int i)
 { return i; }
 
@@ -750,6 +779,21 @@ inline bdd bdd_simplify(const bdd &d, const bdd &b)
 inline bdd bdd_ibuildcubepp(int v, int w, int *a)
 { return bdd_ibuildcube(v,w,a); }
 
+inline bdd bdd_ibuildcubepp2(int v, const int *a)
+{ return bdd_ibuildcube2(v,a); }
+
+inline mintermEnumerator* bdd_init_minterm(const bdd& fun, const bdd& vars)
+{ return bdd_init_minterm(fun.root, vars.root); }
+
+inline int bdd_first_mintermpp(mintermEnumerator* me)
+{ return bdd_first_minterm(me); }
+
+inline int bdd_next_mintermpp(mintermEnumerator* me)
+{ return bdd_next_minterm(me); }
+
+inline void bdd_free_mintermpp(mintermEnumerator* me)
+{ return bdd_free_minterm(me); }
+
 inline bdd bdd_not(const bdd &r)
 { return bdd_not(r.root); }
 
@@ -828,6 +872,12 @@ inline bdd bdd_satoneset(const bdd &r, const bdd &var, const bdd &pol)
 inline bdd bdd_fullsatone(const bdd &r)
 { return bdd_fullsatone(r.root); }
 
+inline bdd bdd_satoneshortest(const bdd &r, unsigned wlow,
+                              unsigned whigh, unsigned wdc)
+{
+  return bdd_satoneshortest(r.root, wlow, whigh, wdc);
+}
+
 inline bdd bdd_satprefix(bdd &r)
 { int ro = r.root; bdd res = bdd_satprefix(&ro); r = bdd(ro); return res; }
 
@@ -859,6 +909,12 @@ inline int* bdd_varprofile(const bdd &r)
 inline double bdd_pathcount(const bdd &r)
 { return bdd_pathcount(r.root); }
 
+inline int bdd_have_common_assignment(const bdd &l, const bdd &r)
+{ return bdd_have_common_assignment(l.root, r.root); }
+
+inline int bdd_is_cube(const bdd &b)
+{ return bdd_is_cube(b.root); }
+
 
    /* I/O extensions */
 
@@ -904,6 +960,10 @@ inline int bdd_addvarblock(const bdd &v, int f)
 #define bdd_nithvar bdd_nithvarpp
 #define bdd_makeset bdd_makesetpp
 #define bdd_ibuildcube bdd_ibuildcubepp
+#define bdd_ibuildcube2 bdd_ibuildcubepp2
+#define bdd_first_minterm bdd_first_mintermpp
+#define bdd_next_minterm bdd_next_mintermpp
+#define bdd_free_minterm bdd_free_mintermpp
 #define bdd_anodecount bdd_anodecountpp
 
 /*=== Inline C++ functions =============================================*/
@@ -1060,6 +1120,95 @@ typedef void (*bddstrmhandler)(std::ostream &, int);
 
 BUDDY_API bddstrmhandler bdd_strm_hook(bddstrmhandler);
 
+/*=== Minterm enumeration ====*/
+
+class minterms_of
+{
+public:
+  class minterm_iterator
+  {
+  public:
+    minterm_iterator(minterms_of* me)
+      : me_(me)
+    {
+    }
+
+    minterm_iterator& operator++();
+    void operator++(int);
+    bool operator==(std::nullptr_t) const;
+    bool operator!=(std::nullptr_t) const;
+    bdd operator*() const;
+  protected:
+    minterms_of* me_;
+  };
+
+  minterms_of(bdd fun, bdd vars)
+  {
+    me_ = bdd_init_minterm(fun, vars);
+  }
+
+  ~minterms_of()
+  {
+    bdd_free_minterm(me_);
+  }
+
+  minterm_iterator begin()
+  {
+    done_ = !bdd_first_minterm(me_);
+    return this;
+  }
+
+  std::nullptr_t end() const
+  {
+    return nullptr;
+  }
+
+  bool done() const
+  {
+    return done_;
+  }
+
+  bdd operator*() const
+  {
+    return bdd_ibuildcube2(me_->nvars, me_->vars);
+  }
+
+  void operator++()
+  {
+    if (!bdd_next_minterm(me_))
+      done_ = true;
+  }
+
+protected:
+  mintermEnumerator* me_;
+  bool done_;
+};
+
+inline minterms_of::minterm_iterator&
+minterms_of::minterm_iterator::operator++()
+{
+  ++*me_;
+  return *this;
+}
+
+inline bool
+minterms_of::minterm_iterator::operator==(std::nullptr_t) const
+{
+  return me_->done();
+}
+
+inline bool
+minterms_of::minterm_iterator::operator!=(std::nullptr_t) const
+{
+  return !me_->done();
+}
+
+inline bdd
+minterms_of::minterm_iterator::operator*() const
+{
+  return **me_;
+}
+
 #endif /* CPLUSPLUS */
 
 #endif /* _BDDX_H */
