@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015-2020 Laboratoire de Recherche et Développement
+// Copyright (C) 2015-2021 Laboratoire de Recherche et Développement
 // de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -1012,6 +1012,13 @@ namespace spot
     }
   }
 
+  bdd acc_cond::acc_code::to_bdd(const bdd* map) const
+  {
+    if (empty())
+      return bddtrue;
+    return to_bdd_rec(&back(), map);
+  }
+
   std::vector<unsigned>
   acc_cond::acc_code::symmetries() const
   {
@@ -1024,7 +1031,7 @@ namespace spot
     std::vector<bdd> r;
     for (unsigned i = 0; r.size() < umax; ++i)
       r.emplace_back(bdd_ithvar(base++));
-    bdd bddcode = to_bdd_rec(&back(), &r[0]);
+    bdd bddcode = to_bdd(&r[0]);
     bdd tmp;
 
     std::vector<unsigned> res(umax);
@@ -1216,7 +1223,7 @@ namespace spot
           }
       }
 
-    bdd res = to_bdd_rec(&back(), &r[0]);
+    bdd res = to_bdd(&r[0]);
 
     if (res == bddtrue)
       return t();
@@ -1285,7 +1292,7 @@ namespace spot
           }
       }
 
-    bdd res = to_bdd_rec(&back(), &r[0]);
+    bdd res = to_bdd(&r[0]);
 
     if (res == bddtrue)
       return t();
@@ -1326,6 +1333,81 @@ namespace spot
     return rescode;
   }
 
+  namespace
+  {
+    static acc_cond::mark_t
+    gather_used_colors(const acc_cond::acc_word* pos)
+    {
+      acc_cond::mark_t res{};
+      auto start = pos - pos->sub.size;
+      do
+        {
+          switch (pos->sub.op)
+            {
+            case acc_cond::acc_op::And:
+            case acc_cond::acc_op::Or:
+              --pos;
+              break;
+            case acc_cond::acc_op::Fin:
+            case acc_cond::acc_op::Inf:
+            case acc_cond::acc_op::FinNeg:
+            case acc_cond::acc_op::InfNeg:
+              res |= pos[-1].mark;
+              pos -= 2;
+              break;
+            }
+        }
+      while (pos > start);
+      return res;
+    }
+  }
+
+  std::vector<std::pair<acc_cond::mark_t, acc_cond::mark_t>>
+  acc_cond::acc_code::useless_colors_patterns() const
+  {
+    // [({y₁,y₂,...,yₙ},{x₁,x₂,...,xₙ}),...]
+    std::vector<std::pair<acc_cond::mark_t, acc_cond::mark_t>>
+      patterns;
+    acc_cond::mark_t used_once = used_once_sets();
+    if (!used_once)
+      return patterns;
+    auto pos = &back();
+    auto end = &front();
+    while (pos > end)
+      {
+        switch (pos->sub.op)
+          {
+          case acc_cond::acc_op::And:
+          case acc_cond::acc_op::Or:
+            {
+              auto expect = pos->sub.op == acc_cond::acc_op::Or ?
+                acc_cond::acc_op::Inf : acc_cond::acc_op::Fin;
+              for (auto p = pos - 1, pe = pos - pos->sub.size;
+                   p >= pe; p -= p->sub.size + 1)
+                if (p->sub.op == expect)
+                  {
+                    acc_cond::mark_t rem{};
+                    for (auto q = pos - 1; q >= pe; q -= q->sub.size + 1)
+                      if (p != q)
+                        rem |= gather_used_colors(q);
+                    rem &= used_once;
+                    if (rem)
+                      patterns.emplace_back(p[-1].mark, rem);
+                  }
+              --pos;
+              break;
+            }
+          case acc_cond::acc_op::Fin:
+          case acc_cond::acc_op::Inf:
+          case acc_cond::acc_op::FinNeg:
+          case acc_cond::acc_op::InfNeg:
+            pos -= 2;
+            break;
+          }
+      }
+    return patterns;
+  }
+
   bool
   acc_cond::acc_code::is_parity_max_equiv(std::vector<int>& permut,
                                           unsigned new_color,
@@ -1563,7 +1645,7 @@ namespace spot
           }
       }
 
-    bdd res = to_bdd_rec(&code_.back(), &r[0]);
+    bdd res = code_.to_bdd(&r[0]);
 
     if (res == bddtrue)
       return {sat, mark_t({})};
@@ -1629,7 +1711,7 @@ namespace spot
           }
       }
 
-    bdd res = to_bdd_rec(&back(), &r[0]);
+    bdd res = to_bdd(&r[0]);
 
     res = bdd_restrict(res, known);
 
@@ -2562,6 +2644,118 @@ namespace spot
     return -1;
   }
 
+  namespace
+  {
+    int has_top_fin(const acc_cond::acc_word* pos, bool top = true)
+    {
+      if (pos->sub.op == acc_cond::acc_op::Fin)
+        {
+          acc_cond::mark_t m = pos[-1].mark;
+          if (top || m.is_singleton())
+            return m.min_set() - 1;
+        }
+      else if (pos->sub.op == acc_cond::acc_op::And)
+        {
+          auto sub = pos - pos->sub.size;
+          do
+            {
+              --pos;
+              if (int f = has_top_fin(pos, false); f >= 0)
+                return f;
+              pos -= pos->sub.size;
+            }
+          while (sub < pos);
+        }
+      else if (top && pos->sub.op == acc_cond::acc_op::Or)
+        {
+          auto sub = pos - pos->sub.size;
+          do
+            {
+              --pos;
+              if (int f = has_top_fin(pos); f >= 0)
+                return f;
+              pos -= pos->sub.size;
+            }
+          while (sub < pos);
+        }
+      return -1;
+    }
+
+    bool uses_fin(const acc_cond::acc_word* pos, acc_cond::mark_t f)
+    {
+      auto sub = pos - pos->sub.size;
+      do
+        {
+          switch (pos->sub.op)
+            {
+            case acc_cond::acc_op::And:
+            case acc_cond::acc_op::Or:
+              --pos;
+              break;
+            case acc_cond::acc_op::Fin:
+              if (pos[-1].mark & f)
+                return true;
+              SPOT_FALLTHROUGH;
+            case acc_cond::acc_op::Inf:
+            case acc_cond::acc_op::InfNeg:
+            case acc_cond::acc_op::FinNeg:
+              pos -= 2;
+              break;
+            }
+        }
+      while (sub < pos);
+      return false;
+    }
+
+    acc_cond::acc_code extract_fin(const acc_cond::acc_word* pos,
+                                   acc_cond::mark_t f)
+    {
+      auto start = pos - pos->sub.size;
+      switch (pos->sub.op)
+        {
+        case acc_cond::acc_op::And:
+        case acc_cond::acc_op::Fin:
+        case acc_cond::acc_op::Inf:
+          return pos;
+        case acc_cond::acc_op::Or:
+          {
+            --pos;
+            auto res = acc_cond::acc_code::f();
+            do
+              {
+                if (uses_fin(pos, f))
+                  {
+                    acc_cond::acc_code tmp(pos);
+                    tmp |= std::move(res);
+                    std::swap(tmp, res);
+                  }
+                pos -= pos->sub.size + 1;
+              }
+            while (pos > start);
+            return res;
+          }
+        case acc_cond::acc_op::FinNeg:
+        case acc_cond::acc_op::InfNeg:
+          SPOT_UNREACHABLE();
+          return {};
+        }
+      SPOT_UNREACHABLE();
+      return {};
+    }
+  }
+
+  std::pair<int, acc_cond::acc_code>
+  acc_cond::acc_code::fin_one_extract() const
+  {
+    if (is_t() || is_f())
+      return {-1, *this};
+    const acc_cond::acc_word* pos = &back();
+    int selected_fin = has_top_fin(pos);
+    if (selected_fin < 0)
+      selected_fin = fin_one();
+    return {selected_fin, extract_fin(pos, {(unsigned) selected_fin})};
+  }
+
   namespace
   {
     bool
@@ -2743,6 +2937,37 @@ namespace spot
     return res;
   }
 
+  acc_cond::mark_t acc_cond::acc_code::inf_unit() const
+  {
+    mark_t res = {};
+    if (empty() || is_f())
+      return res;
+    const acc_cond::acc_word* pos = &back();
+    do
+      {
+        switch (pos->sub.op)
+          {
+          case acc_cond::acc_op::And:
+            --pos;
+            break;
+          case acc_cond::acc_op::Or:
+            pos -= pos->sub.size + 1;
+            break;
+          case acc_cond::acc_op::Fin:
+          case acc_cond::acc_op::InfNeg:
+          case acc_cond::acc_op::FinNeg:
+            pos -= 2;
+            break;
+          case acc_cond::acc_op::Inf:
+            res |= pos[-1].mark;
+            pos -= 2;
+            break;
+          }
+      }
+    while (pos >= &front());
+    return res;
+  }
+
   acc_cond::mark_t acc_cond::acc_code::used_once_sets() const
   {
     mark_t seen = {};
