@@ -76,87 +76,6 @@ namespace spot
 
   namespace
   {
-    twa_graph_ptr merge_identical_marks_here(twa_graph_ptr aut)
-    {
-      // /!\ This assumes that the acceptance condition has been
-      // cleaned up first.  If some mark appears in the acceptance
-      // condition but not in the automaton, the result is undefined.
-
-      auto& acc = aut->acc();
-      auto& c = acc.get_acceptance();
-      acc_cond::mark_t used_in_cond = c.used_sets();
-
-      if (!used_in_cond)
-        return aut;
-
-      unsigned num_sets = acc.num_sets();
-      std::vector<acc_cond::mark_t> always_together(num_sets);
-
-      for (unsigned i = 0; i < num_sets; ++i)
-        if (used_in_cond.has(i))
-          always_together[i] = used_in_cond;
-        else
-          always_together[i] = acc_cond::mark_t({i});
-
-      acc_cond::mark_t previous_a = {};
-      for (auto& t: aut->edges())
-        {
-          acc_cond::mark_t a = t.acc & used_in_cond;
-          if (a == previous_a)
-            continue;
-          previous_a = a;
-          for (unsigned m: a.sets())
-            {
-              acc_cond::mark_t at = always_together[m];
-              acc_cond::mark_t newm = at & a;
-
-              for (unsigned rem: (at - newm).sets())
-                always_together[rem] -= newm;
-
-              always_together[m] = newm;
-            }
-        }
-
-      acc_cond::mark_t to_remove = {};
-      for (unsigned i = 0; i < num_sets; ++i)
-        {
-          auto oldm = always_together[i];
-          if (oldm == acc_cond::mark_t({i}))
-            continue;
-
-          acc_cond::mark_t newm = oldm.lowest();
-          to_remove |= oldm - newm;
-          always_together[i] = newm;
-        }
-      for (auto& t: aut->edges())
-        t.acc -= to_remove;
-
-      // Replace the marks in the acceptance condition
-      auto pos = &c.back();
-      auto end = &c.front();
-      while (pos > end)
-        {
-          switch (pos->sub.op)
-            {
-            case acc_cond::acc_op::And:
-            case acc_cond::acc_op::Or:
-              --pos;
-              break;
-            case acc_cond::acc_op::Fin:
-            case acc_cond::acc_op::Inf:
-            case acc_cond::acc_op::FinNeg:
-            case acc_cond::acc_op::InfNeg:
-              acc_cond::mark_t replace = pos[-1].mark & to_remove;
-              pos[-1].mark -= replace;
-              for (unsigned m: replace.sets())
-                pos[-1].mark |= always_together[m];
-              pos -= 2;
-              break;
-            }
-        }
-      return aut;
-    }
-
     // Remove complementary marks from the acceptance condition.
     acc_cond::acc_code remove_compl_rec(const acc_cond::acc_word* pos,
                                         const std::vector<acc_cond::mark_t>&
@@ -205,7 +124,7 @@ namespace spot
           while (rbegin > rend);
           if (rbegin <= rend)
             return res;
-          // Fin(i) & Inf(i) = f;
+          // Fin(i) & Inf(i) = f;             (also done by unit-fin)
           if (rbegin[-1].mark & seen_fin)
             return std::move(falsecode);
           for (auto m: seen_fin.sets())
@@ -273,6 +192,8 @@ namespace spot
         return aut;
 
       // complement[i] holds sets that appear when set #i does not.
+      // (This is not a strict complement, i.e., the sets in
+      // complement[i] are allowed to occur at the same time as i.)
       unsigned num_sets = acc.num_sets();
       std::vector<acc_cond::mark_t> complement(num_sets);
 
@@ -295,12 +216,8 @@ namespace spot
         {
           prev_acc = tacc;
           for (unsigned m: used_in_cond.sets())
-            {
-              if (tacc.has(m))
-                complement[m] -= tacc;
-              else
-                complement[m] &= tacc;
-            }
+            if (!tacc.has(m))
+              complement[m] &= tacc;
         };
       if (b != e)
         {
@@ -611,21 +528,137 @@ namespace spot
       // Now rewrite the acceptance condition, removing all the "to_kill" terms.
       aut->set_acceptance(aut->num_sets(), acc_rewrite_rec(&acccopy.back()));
     }
+
+    // If X always occurs when Y is present, Inf({X,Y}) can be
+    // replaced by Inf({Y}).
+    //
+    // If X and Y always occur together, then Inf({X}) and Inf({Y})
+    // can be used interchangeably, so we only keep the smallest one.
+    static twa_graph_ptr simplify_included_marks_here(twa_graph_ptr aut)
+    {
+      // Compute included_by[i], the set of colors always included
+      unsigned ns = aut->num_sets();
+      if (ns <= 1)
+        return aut;
+      auto& c = aut->acc().get_acceptance();
+      acc_cond::mark_t used_in_cond = c.used_sets();
+      if (!used_in_cond.has_many())
+        return aut;
+
+      std::vector<acc_cond::mark_t> always_with(ns, used_in_cond);
+      acc_cond::mark_t previous_a{};
+
+      for (auto& e: aut->edges())
+        {
+          // Just avoit the e.acc.sets() loops on marks that we have
+          // just seen.
+          if (e.acc == previous_a)
+            continue;
+          previous_a = e.acc;
+          for (auto c : e.acc.sets())
+            always_with[c] &= e.acc;
+        }
+
+      // For each color n, find the smallest color m that is always used
+      // with n.  If m != n, record that n must be removed (and replaced
+      // by always_together[m]).
+      std::vector<acc_cond::mark_t> always_together(ns);
+      acc_cond::mark_t to_remove{};
+      for (unsigned n = ns - 1; n > 0; --n)
+        {
+          if (!used_in_cond.has(n))
+            continue;
+          always_together[n].set(n);
+          for (unsigned m = 0; m < n; ++m)
+            if (used_in_cond.has(m) && always_with[n] == always_with[m])
+              {
+                always_together[n] = {m};
+                to_remove.set(n);
+                break;
+              }
+        }
+      always_together[0].set(0);
+
+      for (unsigned n = 0; n < ns; ++n)
+        always_with[n].clear(n);
+
+      // Replace the marks in the acceptance condition
+      // We replace each color X in to_remove by always_together[X],
+      // and remove always_with[Y] for each Y in Inf or Fin.
+      auto pos = &c.back();
+      auto end = &c.front();
+      while (pos > end)
+        {
+          switch (pos->sub.op)
+            {
+            case acc_cond::acc_op::And:
+            case acc_cond::acc_op::Or:
+              --pos;
+              break;
+            case acc_cond::acc_op::Fin:
+            case acc_cond::acc_op::Inf:
+            case acc_cond::acc_op::FinNeg:
+            case acc_cond::acc_op::InfNeg:
+              {
+                acc_cond::mark_t colors = pos[-1].mark;
+                acc_cond::mark_t replace = colors & to_remove;
+                colors -= replace;
+                for (unsigned m: replace.sets())
+                  colors |= always_together[m];
+                for (unsigned m: colors.sets())
+                  colors -= always_with[m];
+                pos[-1].mark = colors;
+                pos -= 2;
+                break;
+              }
+            }
+        }
+      return aut;
+    }
+
+
+    // If any subformula of the acceptance condition looks like
+    //   (Inf(y₁)&Inf(y₂)&...&Inf(yₙ)) | f(x₁,...,xₙ)
+    // then for each transition with all colors {y₁,y₂,...,yₙ} we
+    // can remove all the xᵢ that are used only once in the formula.
+    //
+    // See also issue #418.
+    static bool
+    remove_useless_colors_from_edges_here(twa_graph_ptr aut)
+    {
+      // [({y₁,y₂,...,yₙ},{x₁,x₂,...,xₙ}),...]
+      auto patterns = aut->get_acceptance().useless_colors_patterns();
+      if (patterns.empty())
+        return false;
+
+      bool changed = false;
+      for (auto& e: aut->edges())
+        for (auto& p: patterns)
+          if (p.first.subset(e.acc))
+            if (auto nacc = e.acc - p.second; nacc != e.acc)
+              {
+                e.acc = nacc;
+                changed = true;
+              }
+      return changed;
+    }
   }
 
+
   twa_graph_ptr simplify_acceptance_here(twa_graph_ptr aut)
   {
     for (;;)
       {
         cleanup_acceptance_here(aut, false);
-        merge_identical_marks_here(aut);
+        simplify_included_marks_here(aut);
         if (aut->acc().is_generalized_buchi())
           break;
         acc_cond::acc_code old = aut->get_acceptance();
         aut->set_acceptance(aut->acc().unit_propagation());
         simplify_complementary_marks_here(aut);
         fuse_marks_here(aut);
-        if (old == aut->get_acceptance())
+        bool changed = remove_useless_colors_from_edges_here(aut);
+        if (!changed && old == aut->get_acceptance())
           break;
       }
     cleanup_acceptance_here(aut, true);
