@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015-2020 Laboratoire de Recherche et
+// Copyright (C) 2015-2021 Laboratoire de Recherche et
 // Développement de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -94,7 +94,8 @@ namespace spot
     // default constructor
     safra_state();
     safra_state(state_t state_number, bool acceptance_scc = false);
-    safra_state(const safra_build& s, const compute_succs& cs, unsigned& color);
+    safra_state(const safra_build& s, const compute_succs& cs, unsigned& color,
+                unsigned topbrace);
     // Compute successor for transition ap
     safra_state
     compute_succ(const compute_succs& cs, const bdd& ap, unsigned& color) const;
@@ -102,7 +103,7 @@ namespace spot
     merge_redundant_states(const std::vector<std::vector<char>>& implies);
     unsigned
     finalize_construction(const std::vector<int>& buildbraces,
-                          const compute_succs& cs);
+                          const compute_succs& cs, unsigned topbrace);
 
     // each brace points to its parent.
     // braces_[i] is the parent of i
@@ -547,21 +548,6 @@ namespace spot
       return res;
     }
 
-    // Compute a vector of letters from a given support
-    std::vector<bdd>
-    letters(const bdd& allap)
-    {
-      std::vector<bdd> res;
-      bdd all = bddtrue;
-      while (all != bddfalse)
-        {
-          bdd one = bdd_satoneset(all, allap, bddfalse);
-          all -= one;
-          res.emplace_back(one);
-        }
-      return res;
-    }
-
     class safra_support
     {
       const std::vector<bdd>& state_supports;
@@ -578,7 +564,11 @@ namespace spot
           supp &= state_supports[n.first];
         auto i = cache.emplace(supp, std::vector<bdd>());
         if (i.second) // insertion took place
-          i.first->second = letters(supp);
+          {
+            std::vector<bdd>& res = i.first->second;
+            for (bdd one: minterms_of(bddtrue, supp))
+              res.emplace_back(one);
+          }
         return i.first->second;
       }
     };
@@ -593,6 +583,7 @@ namespace spot
     safra_build& ss = cs.ss;
     ss.braces_ = braces_; // copy
     ss.nodes_.clear();
+    unsigned topbrace = braces_.size();
     for (const auto& node: nodes_)
       {
         for (const auto& t: cs.aut->out(node.first))
@@ -612,7 +603,7 @@ namespace spot
               ss.update_succ(node.second, t.dst, t.acc);
           }
       }
-    return safra_state(ss, cs, color);
+    return safra_state(ss, cs, color, topbrace);
   }
 
   // When a node a implies a node b, remove the node a.
@@ -643,7 +634,7 @@ namespace spot
   // Return the emitted color, red or green
   unsigned
   safra_state::finalize_construction(const std::vector<int>& buildbraces,
-                                     const compute_succs& cs)
+                                     const compute_succs& cs, unsigned topbrace)
   {
     unsigned red = -1U;
     unsigned green = -1U;
@@ -698,11 +689,15 @@ namespace spot
             // Step A5 renumber braces
             ++decr;
 
-            // Step A3 emit red
-            red = std::min(red, 2*b);
+            // Any brace above topbrace was added while constructing
+            // this successor, so it should not emit any red.
+            if (b < topbrace)
+              // Step A3 emit red
+              red = std::min(red, 2*b);
           }
         else if (cs.empty_green[b] & is_green)
           {
+            assert(b < topbrace);
             // Step A4 emit green
             green = std::min(green, 2*b+1);
           }
@@ -745,12 +740,13 @@ namespace spot
 
   safra_state::safra_state(const safra_build& s,
                            const compute_succs& cs,
-                           unsigned& color)
+                           unsigned& color,
+                           unsigned topbrace)
   : nodes_(s.nodes_.begin(), s.nodes_.end())
   {
     if (cs.use_simulation)
       merge_redundant_states(cs.implies);
-    color = finalize_construction(s.braces_, cs);
+    color = finalize_construction(s.braces_, cs, topbrace);
   }
 
   bool
@@ -815,7 +811,8 @@ namespace spot
   tgba_determinize(const const_twa_graph_ptr& a,
                    bool pretty_print, bool use_scc,
                    bool use_simulation, bool use_stutter,
-                   const output_aborter* aborter)
+                   const output_aborter* aborter,
+                   int trans_pruning)
   {
     if (!a->is_existential())
       throw std::runtime_error
@@ -833,7 +830,7 @@ namespace spot
       if (use_simulation)
         {
           aut_tmp = spot::scc_filter(aut_tmp);
-          auto aut2 = simulation(aut_tmp, &implications);
+          auto aut2 = simulation(aut_tmp, &implications, trans_pruning);
           if (pretty_print)
             aut2->copy_state_names_from(aut_tmp);
           aut_tmp = aut2;
