@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2019 Laboratoire de Recherche et
+// Copyright (C) 2014-2021 Laboratoire de Recherche et
 // Developpement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -609,6 +609,24 @@ namespace spot
         os << (v1_1 ? "spot." : "spot-") << "rejected-word: \"";
         escape_str(os, *word) << '"' << nl;
       }
+    if (auto player = aut->get_named_prop<std::vector<bool>>("state-player"))
+      {
+        os << (v1_1 ? "spot." : "spot-") << "state-player:";
+        if (player->size() != num_states)
+          throw std::runtime_error("print_hoa(): state-player property has"
+                                   " (" + std::to_string(player->size()) +
+                                   " states but automaton has " +
+                                   std::to_string(num_states));
+        unsigned n = 0;
+        while (n < num_states)
+          {
+            os << ' ' << (*player)[n];
+            ++n;
+            if (newline && n < num_states && (n % 30 == 0))
+              os << "\n                  ";
+          }
+        os << nl;
+      }
 
     // If we want to output implicit labels, we have to
     // fill a vector with all destinations in order.
@@ -692,33 +710,28 @@ namespace spot
         else
           {
             for (auto& t: aut->out(i))
-              {
-                bdd cond = t.cond;
-                while (cond != bddfalse)
-                  {
-                    bdd one = bdd_satoneset(cond, md.all_ap, bddfalse);
-                    cond -= one;
-                    unsigned level = 1;
-                    unsigned pos = 0U;
-                    while (one != bddtrue)
-                      {
-                        bdd h = bdd_high(one);
-                        if (h == bddfalse)
-                          {
-                            one = bdd_low(one);
-                          }
-                        else
-                          {
-                            pos |= level;
-                            one = h;
-                          }
-                        level <<= 1;
-                      }
-                    out[pos] = t.dst;
-                    if (this_acc != Hoa_Acceptance_States)
-                      outm[pos] = t.acc;
-                  }
-              }
+              for (bdd one: minterms_of(t.cond, md.all_ap))
+                {
+                  unsigned level = 1;
+                  unsigned pos = 0U;
+                  while (one != bddtrue)
+                    {
+                      bdd h = bdd_high(one);
+                      if (h == bddfalse)
+                        {
+                          one = bdd_low(one);
+                        }
+                      else
+                        {
+                          pos |= level;
+                          one = h;
+                        }
+                      level <<= 1;
+                    }
+                  out[pos] = t.dst;
+                  if (this_acc != Hoa_Acceptance_States)
+                    outm[pos] = t.acc;
+                }
             unsigned n = out.size();
             for (unsigned i = 0; i < n;)
               {
