@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2008-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2008-2019, 2021 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 // Copyright (C) 2003-2006 Laboratoire d'Informatique de Paris 6
 // (LIP6), département Systèmes Répartis Coopératifs (SRC), Université
@@ -531,7 +531,6 @@ namespace spot
     class ratexp_trad_visitor final
     {
     public:
-      // negated should only be set for constants or atomic properties
       ratexp_trad_visitor(translate_dict& dict, formula to_concat = nullptr)
         : dict_(dict), to_concat_(to_concat)
       {
@@ -720,7 +719,7 @@ namespace spot
                         {
                           if (!tail_computed)
                             {
-                              tail_bdd = recurse(f); // FIXME: inf call!!!
+                              tail_bdd = recurse(f);
                               tail_computed = true;
                             }
                           res |= label & tail_bdd;
@@ -899,11 +898,8 @@ namespace spot
               bdd res_det = bddfalse;
               bdd var_set = bdd_existcomp(bdd_support(res_ndet), dict_.var_set);
               bdd all_props = bdd_existcomp(res_ndet, dict_.var_set);
-              while (all_props != bddfalse)
+              for (bdd label: minterms_of(all_props, var_set))
                 {
-                  bdd label = bdd_satoneset(all_props, var_set, bddtrue);
-                  all_props -= label;
-
                   formula dest =
                     dict_.bdd_to_sere(bdd_appex(res_ndet, label, bddop_and,
                                                 dict_.var_set));
@@ -998,11 +994,8 @@ namespace spot
           // Generate (deterministic) successors
           bdd var_set = bdd_existcomp(bdd_support(res), dict_.var_set);
           bdd all_props = bdd_existcomp(res, dict_.var_set);
-          while (all_props != bddfalse)
+          for (bdd label: minterms_of(all_props, var_set))
             {
-              bdd label = bdd_satoneset(all_props, var_set, bddtrue);
-              all_props -= label;
-
               formula dest =
                 dict_.bdd_to_sere(bdd_appex(res, label, bddop_and,
                                             dict_.var_set));
@@ -1082,7 +1075,8 @@ namespace spot
         }
     }
 
-    // FIXME: use the new tgba::succ() interface
+    // FIXME: use the new twa_graph_ptr interface
+    // with unsigned instead of state*.
     std::tuple<const_twa_graph_ptr,
                const ratexp_to_dfa::namer*,
                const state*>
@@ -1475,11 +1469,8 @@ namespace spot
                 {
                   bdd var_set = bdd_existcomp(bdd_support(f1), dict_.var_set);
                   bdd all_props = bdd_existcomp(f1, dict_.var_set);
-                  while (all_props != bddfalse)
+                  for (bdd label: minterms_of(all_props, var_set))
                     {
-                      bdd label = bdd_satoneset(all_props, var_set, bddtrue);
-                      all_props -= label;
-
                       formula dest =
                         dict_.bdd_to_sere(bdd_appex(f1, label, bddop_and,
                                                     dict_.var_set));
@@ -1559,11 +1550,8 @@ namespace spot
                   bdd var_set = bdd_existcomp(bdd_support(f1), dict_.var_set);
                   bdd all_props = bdd_existcomp(f1, dict_.var_set);
                   bdd missing = !all_props;
-                  while (all_props != bddfalse)
+                  for (bdd label: minterms_of(all_props, var_set))
                     {
-                      bdd label = bdd_satoneset(all_props, var_set, bddtrue);
-                      all_props -= label;
-
                       formula dest =
                         dict_.bdd_to_sere(bdd_appex(f1, label, bddop_and,
                                                     dict_.var_set));
@@ -1793,16 +1781,15 @@ namespace spot
         if (t.has_rational)
           {
             bdd res = bddfalse;
-
-            bdd var_set = bdd_existcomp(bdd_support(t.symbolic), d_.var_set);
-            bdd all_props = bdd_existcomp(t.symbolic, d_.var_set);
-            while (all_props != bddfalse)
+            bdd var_set = bddtrue;
+            bdd all_props = bddtrue;
+            if (d_.exprop)
+              {
+                var_set = bdd_existcomp(bdd_support(t.symbolic), d_.var_set);
+                all_props = bdd_existcomp(t.symbolic, d_.var_set);
+              }
+            for (bdd one_prop_set: minterms_of(all_props, var_set))
               {
-                bdd one_prop_set = bddtrue;
-                if (d_.exprop)
-                  one_prop_set = bdd_satoneset(all_props, var_set, bddtrue);
-                all_props -= one_prop_set;
-
                 minato_isop isop(t.symbolic & one_prop_set);
                 bdd cube;
                 while ((cube = isop.next()) != bddfalse)
@@ -1947,7 +1934,8 @@ namespace spot
   ltl_to_tgba_fm(formula f2, const bdd_dict_ptr& dict,
                  bool exprop, bool symb_merge, bool branching_postponement,
                  bool fair_loop_approx, const atomic_prop_set* unobs,
-                 tl_simplifier* simplifier, bool unambiguous)
+                 tl_simplifier* simplifier, bool unambiguous,
+                 const output_aborter* aborter)
   {
     tl_simplifier* s = simplifier;
 
@@ -2051,6 +2039,13 @@ namespace spot
     dest_map dests;
     while (!formulae_to_translate.empty())
       {
+        if (aborter && aborter->too_large(a))
+          {
+            if (!simplifier)
+              delete s;
+            return nullptr;
+          }
+
         // Pick one formula.
         formula now = *formulae_to_translate.begin();
         formulae_to_translate.erase(formulae_to_translate.begin());
@@ -2102,24 +2097,21 @@ namespace spot
         dests.clear();
 
         // Compute all outgoing arcs.
-
         // If EXPROP is set, we will refine the symbolic
         // representation of the successors for all combinations of
         // the atomic properties involved in the formula.
         // VAR_SET is the set of these properties.
-        bdd var_set = bdd_existcomp(bdd_support(res), d.var_set);
-        // ALL_PROPS is the combinations we have yet to consider.
-        // We used to start with `all_props = bddtrue', but it is
-        // more efficient to start with the set of all satisfiable
-        // variables combinations.
-        bdd all_props = bdd_existcomp(res, d.var_set);
-        while (all_props != bddfalse)
+        // ALL_PROPS is the satisfiable combinations of VAR_SET to consider.
+        bdd var_set = bddtrue;
+        bdd all_props = bddtrue;
+        if (exprop)
           {
-            bdd one_prop_set = bddtrue;
-            if (exprop)
-              one_prop_set = bdd_satoneset(all_props, var_set, bddtrue);
-            all_props -= one_prop_set;
+            var_set = bdd_existcomp(bdd_support(res), d.var_set);
+            all_props = bdd_existcomp(res, d.var_set);
+          }
 
+        for (bdd one_prop_set: minterms_of(all_props, var_set))
+          {
             // Compute prime implicants.
             // The reason we use prime implicants and not bdd_satone()
             // is that we do not want to get any negation in front of Next
