@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2009-2011, 2013-2019 Laboratoire de Recherche et
+// Copyright (C) 2009-2011, 2013-2019, 2021 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 // Copyright (C) 2004 Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -82,7 +82,8 @@ namespace spot
 
   twa_graph_ptr
   tgba_powerset(const const_twa_graph_ptr& aut, power_map& pm, bool merge,
-                const output_aborter* aborter)
+                const output_aborter* aborter,
+                std::vector<unsigned>* accepting_sinks)
   {
     unsigned ns = aut->num_states();
     unsigned nap = aut->ap().size();
@@ -95,12 +96,9 @@ namespace spot
     std::vector<bdd> num2bdd;
     num2bdd.reserve(1UL << nap);
     std::map<bdd, unsigned, bdd_less_than> bdd2num;
-    bdd all = bddtrue;
     bdd allap = aut->ap_vars();
-    while (all != bddfalse)
+    for (bdd one: minterms_of(bddtrue, allap))
       {
-        bdd one = bdd_satoneset(all, allap, bddfalse);
-        all -= one;
         bdd2num.emplace(one, num2bdd.size());
         num2bdd.emplace_back(one);
       }
@@ -182,16 +180,11 @@ namespace spot
         for (unsigned i = 0; i < nc; ++i)
           bv->at(base + i).clear_all();
       for (auto& t: aut->out(src))
-        {
-          bdd all = t.cond;
-          while (all != bddfalse)
-            {
-              bdd one = bdd_satoneset(all, allap, bddfalse);
-              all -= one;
-              unsigned num = bdd2num[one];
-              bv->at(base + num).set(t.dst);
-            }
-        }
+        for (bdd one: minterms_of(t.cond, allap))
+          {
+            unsigned num = bdd2num[one];
+            bv->at(base + num).set(t.dst);
+          }
 
       assert(idx == lru.begin()->first);
       return idx;
@@ -207,6 +200,23 @@ namespace spot
     auto res = make_twa_graph(aut->get_dict());
     res->copy_ap_of(aut);
 
+    bitvect* acc_sinks = nullptr;
+    if (accepting_sinks)
+      {
+        acc_sinks = make_bitvect(ns);
+        for (unsigned s: *accepting_sinks)
+          acc_sinks->set(s);
+        toclean.emplace_back(acc_sinks);
+
+        // The accepting sink is the first registered state by
+        // convention.
+        power_state ps = bv_to_ps(acc_sinks);
+        unsigned num = res->new_state();
+        seen[acc_sinks] = num;
+        assert(pm.map_.size() == num);
+        pm.map_.emplace_back(std::move(ps));
+      }
+
     {
       unsigned init_num = aut->get_init_state_number();
       auto bvi = make_bitvect(ns);
@@ -239,6 +249,8 @@ namespace spot
             auto dst = &om->at(c);
             if (dst->is_fully_clear())
               continue;
+            if (acc_sinks && dst->intersects(*acc_sinks))
+              *dst = *acc_sinks;
             auto i = seen.find(dst);
             unsigned dst_num;
             if (i != seen.end())
@@ -274,10 +286,11 @@ namespace spot
 
   twa_graph_ptr
   tgba_powerset(const const_twa_graph_ptr& aut,
-                const output_aborter* aborter)
+                const output_aborter* aborter,
+                std::vector<unsigned>* accepting_sinks)
   {
     power_map pm;
-    return tgba_powerset(aut, pm, true, aborter);
+    return tgba_powerset(aut, pm, true, aborter, accepting_sinks);
   }
 
 
