@@ -1,5 +1,5 @@
 # -*- mode: python; coding: utf-8 -*-
-# Copyright (C) 2018 Laboratoire de Recherche et
+# Copyright (C) 2018-2021 Laboratoire de Recherche et
 # Développement de l'Epita
 #
 # This file is part of Spot, a model checking library.
@@ -18,6 +18,20 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import spot
+import buddy
+
+# CPython use reference counting, so that automata are destructed
+# when we expect them to be.   However other implementations like
+# PyPy may call destructors latter, causing different output.
+from platform import python_implementation
+if python_implementation() == 'CPython':
+    def gcollect():
+        pass
+else:
+    import gc
+    def gcollect():
+        gc.collect()
+
 
 
 def incl(a, b):
@@ -28,42 +42,50 @@ def equiv(a, b):
     return incl(a, b) and incl(b, a)
 
 
-def do_split(f, in_list):
+def do_split(f, out_list):
     aut = spot.translate(f)
-    inputs = spot.buddy.bddtrue
-    for a in in_list:
-        inputs &= spot.buddy.bdd_ithvar(aut.get_dict().varnum(spot.formula(a)))
-    s = spot.split_2step(aut, inputs)
+    outputs = buddy.bddtrue
+    for a in out_list:
+        outputs &= buddy.bdd_ithvar(aut.register_ap(a))
+    s = spot.split_2step(aut, outputs, False)
     return aut, s
 
-
-aut, s = do_split('(FG !a) <-> (GF b)', ['a'])
+aut, s = do_split('(FG !a) <-> (GF b)', ['b'])
 assert equiv(aut, spot.unsplit_2step(s))
 
-aut, s = do_split('GFa && GFb', ['a'])
-assert equiv(aut, spot.unsplit_2step(s))
-assert s.to_str() == """HOA: v1
-States: 3
-Start: 0
-AP: 2 "a" "b"
-acc-name: generalized-Buchi 2
-Acceptance: 2 Inf(0)&Inf(1)
-properties: trans-labels explicit-labels trans-acc complete
-properties: deterministic
---BODY--
-State: 0
-[0] 1
-[!0] 2
-State: 1
-[!1] 0 {0}
-[1] 0 {0 1}
-State: 2
-[!1] 0
-[1] 0 {1}
---END--"""
+del aut
+del s
+gcollect()
 
-aut, s = do_split('! ((G (req -> (F ack))) && (G (go -> (F grant))))', ['go',
-                                                                        'req'])
+aut, s = do_split('GFa && GFb', ['b'])
+assert equiv(aut, spot.unsplit_2step(s))
+# FIXME see below
+# assert str_diff("""HOA: v1
+# States: 3
+# Start: 0
+# AP: 2 "a" "b"
+# acc-name: generalized-Buchi 2
+# Acceptance: 2 Inf(0)&Inf(1)
+# properties: trans-labels explicit-labels trans-acc complete
+# properties: deterministic
+# spot-state-player: 0 1 1
+# --BODY--
+# State: 0
+# [0] 1
+# [!0] 2
+# State: 1
+# [!1] 0 {0}
+# [1] 0 {0 1}
+# State: 2
+# [!1] 0
+# [1] 0 {1}
+# --END--""", s.to_str() )
+
+del aut
+del s
+gcollect()
+
+aut, s = do_split('! ((G (req -> (F ack))) && (G (go -> (F grant))))', ['ack'])
 assert equiv(aut, spot.unsplit_2step(s))
 # FIXME s.to_str() is NOT the same on Debian stable and on Debian unstable
 #       we should investigate this
@@ -102,6 +124,11 @@ assert equiv(aut, spot.unsplit_2step(s))
 # [!3] 2
 # --END--"""
 
+del aut
+del s
+gcollect()
+
 aut, s = do_split('((G (((! g_0) || (! g_1)) && ((r_0 && (X r_1)) -> (F (g_0 \
-    && g_1))))) && (G (r_0 -> F g_0))) && (G (r_1 -> F g_1))', ['r_0', 'r_1'])
+    && g_1))))) && (G (r_0 -> F g_0))) && (G (r_1 -> F g_1))',
+                  ['g_0', 'g_1'])
 assert equiv(aut, spot.unsplit_2step(s))
