@@ -1,6 +1,6 @@
 # -*- mode: python; coding: utf-8 -*-
-# Copyright (C) 2016-2018, 2020  Laboratoire de Recherche et Développement
-# de l'Epita
+# Copyright (C) 2016-2018, 2020-2021  Laboratoire de Recherche
+# et Développement de l'Epita
 #
 # This file is part of Spot, a model checking library.
 #
@@ -19,6 +19,12 @@
 
 import spot
 
+# CPython use reference counting, so that automata are destructed
+# when we expect them to be.   However other implementations like
+# PyPy may call destructors latter, causing different output.
+from platform import python_implementation
+is_cpython = python_implementation() == 'CPython'
+
 # Test 1.
 aut = spot.automaton("""
 HOA: v1
@@ -232,7 +238,7 @@ State: 0 "F(Gp3|GFp2)"
 """)
 
 exp = """HOA: v1
-States: 4
+States: 3
 Start: 0
 AP: 2 "p3" "p2"
 acc-name: Buchi
@@ -246,14 +252,8 @@ State: 0
 State: 1 {0}
 [!1] 0
 [1] 1
-[0&!1] 2
-[0&1] 3
 State: 2 {0}
 [0&!1] 2
-[0&1] 3
-State: 3 {0}
-[0&!1] 2
-[0&1] 3
 --END--"""
 
 res = spot.remove_fin(aut)
@@ -371,7 +371,10 @@ State: 7
 --END--"""
 
 res = spot.remove_fin(aut)
-assert(res.to_str('hoa') == exp)
+if is_cpython:
+    assert(res.to_str('hoa') == exp)
+else:
+    assert(res.equivalent_to(spot.automaton(exp)))
 
 # Test 9.
 aut = spot.automaton("""
@@ -407,7 +410,10 @@ State: 1
 --END--"""
 
 res = spot.remove_fin(aut)
-assert(res.to_str('hoa') == exp)
+if is_cpython:
+    assert(res.to_str('hoa') == exp)
+else:
+    assert(res.equivalent_to(spot.automaton(exp)))
 
 # Test 10.
 aut = spot.automaton("""
@@ -446,7 +452,10 @@ State: 2 {0}
 --END--"""
 
 res = spot.remove_fin(aut)
-assert(res.to_str('hoa') == exp)
+if is_cpython:
+    assert(res.to_str('hoa') == exp)
+else:
+    assert(res.equivalent_to(spot.automaton(exp)))
 
 # Test 11.
 aut = spot.automaton("""
@@ -483,7 +492,10 @@ State: 1
 --END--"""
 
 res = spot.remove_fin(aut)
-assert(res.to_str('hoa') == exp)
+if is_cpython:
+    assert(res.to_str('hoa') == exp)
+else:
+    assert(res.equivalent_to(spot.automaton(exp)))
 
 # Different order for rabin_to_buchi_if_realizable() due to merge_edges() not
 # being called.  This is on purpose: the edge order should match exactly the
@@ -505,7 +517,10 @@ State: 1
 [!0] 0 {0}
 --END--"""
 res = spot.rabin_to_buchi_if_realizable(aut)
-assert(res.to_str('hoa') == exp2)
+if is_cpython:
+    assert(res.to_str('hoa') == exp2)
+else:
+    assert(res.equivalent_to(spot.automaton(exp2)))
 
 # Test 12.
 aut = spot.automaton("""
@@ -542,7 +557,6 @@ State: 0
 State: 1
 [!0] 0
 [0] 1
-[!0] 3
 State: 2 {0}
 [!0] 2
 State: 3 {0}
@@ -550,7 +564,10 @@ State: 3 {0}
 --END--"""
 
 res = spot.remove_fin(aut)
-assert(res.to_str('hoa') == exp)
+if is_cpython:
+    assert(res.to_str('hoa') == exp)
+else:
+    assert(res.equivalent_to(spot.automaton(exp)))
 
 # Test 13.
 aut = spot.automaton("""
@@ -585,15 +602,22 @@ properties: trans-labels explicit-labels trans-acc complete
 properties: deterministic
 --BODY--
 State: 0
-[!1] 0 {0}
-[1] 1
+[0&1] 1 {0}
+[0&!1] 0 {0}
+[!0&1] 1
+[!0&!1] 0 {0}
 State: 1
-[!1] 0 {0}
-[1] 1 {0}
+[0&1] 1 {0}
+[0&!1] 0 {0}
+[!0&1] 1 {0}
+[!0&!1] 0 {0}
 --END--"""
 
 res = spot.remove_fin(aut)
-assert(res.to_str('hoa') == exp)
+if is_cpython:
+    assert(res.to_str('hoa') == exp)
+else:
+    assert(res.equivalent_to(spot.automaton(exp)))
 
 # rabin_to_buchi_if_realizable() does not call merge_edges() on purpose: the
 # edge order should match exactly the original automaton.
@@ -619,7 +643,10 @@ State: 1
 --END--"""
 
 res = spot.rabin_to_buchi_if_realizable(aut)
-assert(res.to_str('hoa') == exp2)
+if is_cpython:
+    assert(res.to_str('hoa') == exp2)
+else:
+    assert(res.equivalent_to(spot.automaton(exp2)))
 
 # Test 14.
 aut = spot.automaton("""
@@ -648,11 +675,13 @@ State: 0
 [!0] 0
 [0] 0 {0}
 [!0&!1] 1
-[0&!1] 1 {0}
 State: 1
-[!1] 1 {0}
+[!0&!1] 1 {0}
 --END--"""
 
 res = spot.remove_fin(aut)
-assert(res.to_str('hoa') == exp)
+if is_cpython:
+    assert(res.to_str('hoa') == exp)
+else:
+    assert(res.equivalent_to(spot.automaton(exp)))
 assert spot.rabin_to_buchi_if_realizable(aut) is None
